package at.ac.uibk.gitsearch.service.dto;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonValue;

public class SearchResultsDTO {

	@JsonFormat(shape = JsonFormat.Shape.OBJECT)
	public enum ExerciseType {
		PROGRAMMING_EXERCISE("programming exercise"), EXERCISE("exercise"), COLLECTION("collection"), OTHER("other");

		ExerciseType(String externalName) {
			this.externalName = externalName;
		}

		private String externalName;

		@JsonValue
		public String getExternalName() {
			return externalName;
		}
	}
	
	public static class PluginActionInfo {
		String plugin;
		String action;
		String commandName;
		
		public PluginActionInfo(String plugin, String action, String commandName) {
			super();
			this.plugin = plugin;
			this.action = action;
			this.commandName = commandName;
		}
		
		public PluginActionInfo() {
			// JSON
		}
		/**
		 * @return the plugin
		 */
		public String getPlugin() {
			return plugin;
		}
		/**
		 * @param plugin the plugin to set
		 */
		public void setPlugin(String plugin) {
			this.plugin = plugin;
		}
		/**
		 * @return the action
		 */
		public String getAction() {
			return action;
		}
		/**
		 * @param action the action to set
		 */
		public void setAction(String action) {
			this.action = action;
		}
		/**
		 * @return the commandName
		 */
		public String getCommandName() {
			return commandName;
		}
		/**
		 * @param commandName the commandName to set
		 */
		public void setCommandName(String commandName) {
			this.commandName = commandName;
		}
		
		
	}

	public static class Person {
		private String name;
		private String affiliation;
		private String email;

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public String getAffiliation() {
			return affiliation;
		}

		public void setAffiliation(String affiliation) {
			this.affiliation = affiliation;
		}

		public String getEmail() {
			return email;
		}

		public void setEmail(String email) {
			this.email = email;
		}

		@Override
		public String toString() {
			return "Person: { name: " + this.name + ", affiliation: " + this.affiliation + ", email: " + this.email
					+ "}";
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((affiliation == null) ? 0 : affiliation.hashCode());
			result = prime * result + ((email == null) ? 0 : email.hashCode());
			result = prime * result + ((name == null) ? 0 : name.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			Person other = (Person) obj;
			if (affiliation == null) {
				if (other.affiliation != null)
					return false;
			} else if (!affiliation.equals(other.affiliation))
				return false;
			if (email == null) {
				if (other.email != null)
					return false;
			} else if (!email.equals(other.email))
				return false;
			if (name == null) {
				if (other.name != null)
					return false;
			} else if (!name.equals(other.name))
				return false;
			return true;
		}
	}
	
	public enum GitProjectVisibility {
		PRIVATE("private"), PUBLIC("public"), INTERNAL("internal");
		
		private String externalName;

		private GitProjectVisibility(String externalName) {
			this.externalName = externalName;
		}

		@JsonValue
		public String getExternalName() {
			return externalName;
		}

	}

	public static class GitProject {
		private int project_id;
		private Date last_activity_at;
		private String url;
		private GitProjectVisibility visibility; 
		private String namespace;
		public int getProject_id() {
			return project_id;
		}

		public void setProject_id(int project_id) {
			this.project_id = project_id;
		}

		public int getProjectId() {
			return project_id;
		}

		public void setProjectId(int project_id) {
			this.project_id = project_id;
		}

		/**
		 * @return the last_activity_at
		 */
		public Date getLast_activity_at() {
			return last_activity_at;
		}

		/**
		 * @param last_activity_at the last_activity_at to set
		 */
		public void setLast_activity_at(Date last_activity_at) {
			this.last_activity_at = last_activity_at;
		}

		/**
		 * @return the url
		 */
		public String getUrl() {
			return url;
		}

		/**
		 * @param url the url to set
		 */
		public void setUrl(String url) {
			this.url = url;
		}

		/**
		 * @return the visibility
		 */
		public GitProjectVisibility getVisibility() {
			return visibility;
		}

		/**
		 * @param visibility the visibility to set
		 */
		public void setVisibility(GitProjectVisibility visibility) {
			this.visibility = visibility;
		}

		/**
		 * @return the namespace
		 */
		public String getNamespace() {
			return namespace;
		}

		/**
		 * @param namespace the namespace to set
		 */
		public void setNamespace(String namespace) {
			this.namespace = namespace;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + project_id;
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			GitProject other = (GitProject) obj;
			if (project_id != other.project_id)
				return false;
			return true;
		}
	}

	public static class SearchResultDTO {

		public SearchResultDTO() {
			// default constructor
		}

		private String metadataVersion; // just for YAML testdata reader
		private String title;
		private String identifier;
		private String version;
		private String structure;

		private String description;
		private ExerciseType type;
		private String license;
		private String[] keyword;

		private String[] format;
		private String[] programmingLanguage;
		private String[] language;
		private String status;

		private String educationLevel;

		private String gitURL;

		@JsonIgnore
		private String audience;

		private String timeRequired;

		private GitProject project;

		@JsonIgnore
		private String[] collectionContent;

		public String getTimeRequired() {
			return timeRequired;
		}

		public void setTimeRequired(String timeRequired) {
			this.timeRequired = timeRequired;
		}

		private Person[] creator;
		private Person[] publisher;
		private Person[] contributor;

		private boolean deprecated;

		private String difficulty;
		private String[] source;

		private String[] requires;
		private String image;
		
		private List<PluginActionInfo> supportedActions = new ArrayList<>(2);

		public String getIdentifier() {
			return identifier;
		}

		public void setIdentifier(String identifier) {
			this.identifier = identifier;
		}

		public String getVersion() {
			return version;
		}

		public void setVersion(String version) {
			this.version = version;
		}

		public ExerciseType getType() {
			return type;
		}

		public void setType(ExerciseType type) {
			this.type = type;
		}

		public String[] getLanguage() {
			return language;
		}

		public void setLanguage(String[] language) {
			this.language = language;
		}

		public String getStatus() {
			return status;
		}

		public void setStatus(String status) {
			this.status = status;
		}

		public String getEducationLevel() {
			return educationLevel;
		}

		public void setEducationLevel(String educationLevel) {
			this.educationLevel = educationLevel;
		}

		public String getAudience() {
			return audience;
		}

		public void setAudience(String audience) {
			this.audience = audience;
		}

		public Person[] getCreator() {
			return creator;
		}

		public void setCreator(Person[] creator) {
			this.creator = creator;
		}

		public Person[] getPublisher() {
			return publisher;
		}

		public void setPublisher(Person[] publisher) {
			this.publisher = publisher;
		}

		public Person[] getContributor() {
			return contributor;
		}

		public void setContributor(Person[] contributor) {
			this.contributor = contributor;
		}

		public boolean isDeprecated() {
			return deprecated;
		}

		public void setDeprecated(boolean deprecated) {
			this.deprecated = deprecated;
		}

		public String getDifficulty() {
			return difficulty;
		}

		public void setDifficulty(String difficulty) {
			this.difficulty = difficulty;
		}

		public String[] getSource() {
			return source;
		}

		public void setSource(String[] source) {
			this.source = source;
		}

		public String[] getRequires() {
			return requires;
		}

		public void setRequires(String[] requires) {
			this.requires = requires;
		}

		public String getImage() {
			return image;
		}

		public void setImage(String image) {
			this.image = image;
		}

		private String repositoryURL;

		public String getMetadataVersion() {
			return metadataVersion;
		}

		public void setMetadataVersion(String metadataVersion) {
			this.metadataVersion = metadataVersion;
		}

		public String getTitle() {
			return title;
		}

		public void setTitle(String title) {
			this.title = title;
		}

		public String getDescription() {
			return description;
		}

		public void setDescription(String description) {
			this.description = description;
		}

		public String getLicense() {
			return license;
		}

		public void setLicense(String license) {
			this.license = license;
		}

		public String[] getKeyword() {
			return keyword;
		}

		public void setKeyword(String[] keyword) {
			this.keyword = keyword;
		}

		public String[] getFormat() {
			return format;
		}

		public void setFormat(String[] format) {
			this.format = format;
		}

		public String[] getProgrammingLanguage() {
			return programmingLanguage;
		}

		public void setProgrammingLanguage(String[] programmingLanguage) {
			this.programmingLanguage = programmingLanguage;
		}

		public String getRepositoryURL() {
			return repositoryURL;
		}

		public void setRepositoryURL(String repositoryURL) {
			this.repositoryURL = repositoryURL;
		}

		public String getStructure() {
			return structure;
		}

		public void setStructure(String structure) {
			this.structure = structure;
		}

		public String getGitURL() {
			return gitURL;
		}

		public void setGitURL(String gitURL) {
			this.gitURL = gitURL;
		}

		public GitProject getProject() {
			return project;
		}

		public void setProject(GitProject project) {
			this.project = project;
		}

		/**
		 * @return the supportedActions
		 */
		public List<PluginActionInfo> getSupportedActions() {
			return supportedActions;
		}

		/**
		 * @param supportedActions the supportedActions to set
		 */
		public void setSupportedActions(List<PluginActionInfo> supportedActions) {
			this.supportedActions = supportedActions;
		}

		/**
		 * clone constructor
		 *
		 * @param toClone
		 */
		public SearchResultDTO(SearchResultDTO toClone) {
			super();
			this.metadataVersion = toClone.metadataVersion;
			this.title = toClone.title;
			this.identifier = toClone.identifier;
			this.version = toClone.version;
			this.structure = toClone.structure;
			this.description = toClone.description;
			this.type = toClone.type;
			this.license = toClone.license;
			this.keyword = toClone.keyword;
			this.format = toClone.format;
			this.programmingLanguage = toClone.programmingLanguage;
			this.language = toClone.language;
			this.status = toClone.status;
			this.educationLevel = toClone.educationLevel;
			this.gitURL = toClone.gitURL;
			this.audience = toClone.audience;
			this.timeRequired = toClone.timeRequired;
			this.collectionContent = toClone.collectionContent;
			this.creator = toClone.creator;
			this.publisher = toClone.publisher;
			this.contributor = toClone.contributor;
			this.deprecated = toClone.deprecated;
			this.difficulty = toClone.difficulty;
			this.source = toClone.source;
			this.requires = toClone.requires;
			this.image = toClone.image;
			this.repositoryURL = toClone.repositoryURL;
		}

		@Override
		public String toString() {
			return "SearchResultDTO [audience=" + audience + ", collectionContent=" + collectionContent
					+ ", contributor=" + Arrays.toString(contributor) + ", creator=" + Arrays.toString(creator)
					+ ", deprecated=" + deprecated + ", description=" + description + ", difficulty=" + difficulty
					+ ", educationLevel=" + educationLevel + ", format=" + format + ", gitURL=" + gitURL
					+ ", identifier=" + identifier + ", image=" + image + ", keyword=" + Arrays.toString(keyword)
					+ ", language=" + Arrays.toString(language) + ", license=" + license + ", metadataVersion="
					+ metadataVersion + ", programmingLanguage=" + Arrays.toString(programmingLanguage) + ", publisher="
					+ Arrays.toString(publisher) + ", repositoryURL=" + repositoryURL + ", requires=" + requires
					+ ", source=" + Arrays.toString(source) + ", status=" + status + ", structure=" + structure
					+ ", timeRequired=" + timeRequired + ", title=" + title + ", type=" + type + ", version=" + version
					+ "]";
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((audience == null) ? 0 : audience.hashCode());
			result = prime * result + Arrays.hashCode(collectionContent);
			result = prime * result + Arrays.hashCode(contributor);
			result = prime * result + Arrays.hashCode(creator);
			result = prime * result + (deprecated ? 1231 : 1237);
			result = prime * result + ((description == null) ? 0 : description.hashCode());
			result = prime * result + ((difficulty == null) ? 0 : difficulty.hashCode());
			result = prime * result + ((educationLevel == null) ? 0 : educationLevel.hashCode());
			result = prime * result + Arrays.hashCode(format);
			result = prime * result + ((gitURL == null) ? 0 : gitURL.hashCode());
			result = prime * result + ((identifier == null) ? 0 : identifier.hashCode());
			result = prime * result + ((image == null) ? 0 : image.hashCode());
			result = prime * result + Arrays.hashCode(keyword);
			result = prime * result + Arrays.hashCode(language);
			result = prime * result + ((license == null) ? 0 : license.hashCode());
			result = prime * result + ((metadataVersion == null) ? 0 : metadataVersion.hashCode());
			result = prime * result + Arrays.hashCode(programmingLanguage);
			result = prime * result + ((project == null) ? 0 : project.hashCode());
			result = prime * result + Arrays.hashCode(publisher);
			result = prime * result + ((repositoryURL == null) ? 0 : repositoryURL.hashCode());
			result = prime * result + Arrays.hashCode(requires);
			result = prime * result + Arrays.hashCode(source);
			result = prime * result + ((status == null) ? 0 : status.hashCode());
			result = prime * result + ((structure == null) ? 0 : structure.hashCode());
			result = prime * result + ((timeRequired == null) ? 0 : timeRequired.hashCode());
			result = prime * result + ((title == null) ? 0 : title.hashCode());
			result = prime * result + ((type == null) ? 0 : type.hashCode());
			result = prime * result + ((version == null) ? 0 : version.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			SearchResultDTO other = (SearchResultDTO) obj;
			if (audience == null) {
				if (other.audience != null)
					return false;
			} else if (!audience.equals(other.audience))
				return false;
			if (!Arrays.equals(collectionContent, other.collectionContent))
				return false;
			if (!Arrays.equals(contributor, other.contributor))
				return false;
			if (!Arrays.equals(creator, other.creator))
				return false;
			if (deprecated != other.deprecated)
				return false;
			if (description == null) {
				if (other.description != null)
					return false;
			} else if (!description.equals(other.description))
				return false;
			if (difficulty == null) {
				if (other.difficulty != null)
					return false;
			} else if (!difficulty.equals(other.difficulty))
				return false;
			if (educationLevel == null) {
				if (other.educationLevel != null)
					return false;
			} else if (!educationLevel.equals(other.educationLevel))
				return false;
			if (!Arrays.equals(format, other.format))
				return false;
			if (gitURL == null) {
				if (other.gitURL != null)
					return false;
			} else if (!gitURL.equals(other.gitURL))
				return false;
			if (identifier == null) {
				if (other.identifier != null)
					return false;
			} else if (!identifier.equals(other.identifier))
				return false;
			if (image == null) {
				if (other.image != null)
					return false;
			} else if (!image.equals(other.image))
				return false;
			if (!Arrays.equals(keyword, other.keyword))
				return false;
			if (!Arrays.equals(language, other.language))
				return false;
			if (license == null) {
				if (other.license != null)
					return false;
			} else if (!license.equals(other.license))
				return false;
			if (metadataVersion == null) {
				if (other.metadataVersion != null)
					return false;
			} else if (!metadataVersion.equals(other.metadataVersion))
				return false;
			if (!Arrays.equals(programmingLanguage, other.programmingLanguage))
				return false;
			if (project == null) {
				if (other.project != null)
					return false;
			} else if (!project.equals(other.project))
				return false;
			if (!Arrays.equals(publisher, other.publisher))
				return false;
			if (repositoryURL == null) {
				if (other.repositoryURL != null)
					return false;
			} else if (!repositoryURL.equals(other.repositoryURL))
				return false;
			if (!Arrays.equals(requires, other.requires))
				return false;
			if (!Arrays.equals(source, other.source))
				return false;
			if (status == null) {
				if (other.status != null)
					return false;
			} else if (!status.equals(other.status))
				return false;
			if (structure == null) {
				if (other.structure != null)
					return false;
			} else if (!structure.equals(other.structure))
				return false;
			if (timeRequired == null) {
				if (other.timeRequired != null)
					return false;
			} else if (!timeRequired.equals(other.timeRequired))
				return false;
			if (title == null) {
				if (other.title != null)
					return false;
			} else if (!title.equals(other.title))
				return false;
			if (type != other.type)
				return false;
			if (version == null) {
				if (other.version != null)
					return false;
			} else if (!version.equals(other.version))
				return false;
			return true;
		}
	}

	private List<SearchResultDTO> searchResult;

	/**
	 *
	 * @return the list of search results on this page, starting with pageStartIndex
	 */
	public List<SearchResultDTO> getSearchResult() {
		return searchResult;
	}

	public void setSearchResult(List<SearchResultDTO> searchResult) {
		this.searchResult = searchResult;
	}

	long hitCount;

	long pageStartIndex;

	public long getPageStartIndex() {
		return pageStartIndex;
	}

	public SearchResultsDTO(List<SearchResultDTO> searchResult, long hitCount, long pageStartIndex) {
		this.searchResult = searchResult;
		this.hitCount = hitCount;
		this.pageStartIndex = pageStartIndex;
	}

	public SearchResultsDTO() {

	}

	/**
	 * returns the number of all hits.
	 * 
	 * @return
	 */
	public long getHitCount() {
		return hitCount;
	}

	public void setHitCount(long hitCount) {
		this.hitCount = hitCount;
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder();
		sb.append("SearchResultsDTO:");
		sb.append(System.lineSeparator());
		sb.append("hit count: ");
		sb.append(this.hitCount);
		sb.append(System.lineSeparator());
		sb.append("page start index: ");
		sb.append(this.pageStartIndex);
		sb.append(System.lineSeparator());
		sb.append("list of SearchResultsDTOs:");
		sb.append(System.lineSeparator());
		for (SearchResultDTO result : this.searchResult) {
			sb.append(result);
			sb.append(System.lineSeparator());
			sb.append(System.lineSeparator());
		}
		return sb.toString();
	}
}
