package at.ac.uibk.gitsearch.web.rest;

import java.io.IOException;
import java.io.InputStream;

import javax.websocket.server.PathParam;

import org.codeability.sharing.plugins.api.ShoppingBasket;
import org.gitlab4j.api.GitLabApiException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import at.ac.uibk.gitsearch.es.model.DocumentInfo;
import at.ac.uibk.gitsearch.service.ShoppingBasketService;
import at.ac.uibk.gitsearch.service.ShoppingBasketService.ShoppingBasketInfoDTO;
import at.ac.uibk.gitsearch.service.ShoppingBasketService.ShoppingBasketRedirectInfoDTO;

/**
 * REST controller for managing {@link DocumentInfo}.
 */
@RestController
@RequestMapping("/api")
@Transactional
public class PluginInterfaceResource {

	private final Logger log = LoggerFactory.getLogger(PluginInterfaceResource.class);

	@Autowired
	private ShoppingBasketService basketService;
	
	/**
	 * assigns a shopping basket on server, and provides a redirect link together with the shopping basket token.
	 * @param basketInfo
	 * @return
	 */
    @PostMapping("/pluginIF/getPluginRedirectInfos") 
	public ShoppingBasketRedirectInfoDTO getRedirectInfos(@RequestBody ShoppingBasketInfoDTO basketInfo) {
    	final String baseUrl = 
    			ServletUriComponentsBuilder.fromCurrentContextPath().build().toUriString();
    	return basketService.getRedirectInfo(basketInfo, baseUrl);
    }
    
    
	
    /**
     * {@code SEARCH  /search/page-details} : search for the searchResults corresponding
     *  to the query.
     *
	 * @param query the query of the searchResult search.
	 * @return the result of the search.
     */
    @GetMapping("/pluginIF/v0.1/basket/{basketToken}") 
    public ShoppingBasket searchPageDetails(
    		@PathParam("basketToken") String basketToken) throws IOException {
        log.debug("REST request for basket  {}", basketToken);
        return basketService.getBasket(basketToken);
    }

    @GetMapping("/pluginIF/v0.1/basket/{basketToken}/repository/{exerciseId}") 
    public ResponseEntity<?> getRepositoryZip(@PathVariable String basketToken, @PathVariable int exerciseId) throws IOException {
    	InputStream zip;
		try {
			zip = basketService.getRepositoryZip(basketToken, exerciseId);
		} catch (GitLabApiException e) {
			return ResponseEntity.unprocessableEntity().body(e.getMessage());
		}
    	InputStreamResource resource = new InputStreamResource(zip);
        return ResponseEntity.ok().contentType(org.springframework.http.MediaType.APPLICATION_OCTET_STREAM).header("filename", "Repository"+exerciseId).body(resource);

    }


}
